//**********************************************************
// Program: EIA_EVmatch.java
//
// Author: Charles Marquet
//
// Description: This program reads the EIA-EPA and EV data files and tries to match generators.
//              The program also assigns ever EIA-EPA unit to a bus in the network
//              
// Input: The EIA-EPA data set, from the match.java program   
//        The full EV dataset
//        The generator-bus correspondance file
//        A file with the sum of expected capacity at each bus
//
// Output: The EIA-EV_Gens_1.csv file, with all EIA-EPA units and the matched EV units.
//
// Need the free csv reader/writer from: http://sourceforge.net/projects/javacsv/
//*********************************************************

import java.io.*;
import java.lang.String;
import java.lang.Math;
import java.lang.Integer;
import com.csvreader.CsvReader;
import com.csvreader.CsvWriter;
import java.util.*;

public class EIA_EVmatch {
  //these are the input file names
  //Input: EIA-EPA-stacks matched file
  private static final String ADDRESS_FILE_EIA="match_fips.csv";  //"match-damages_NE.csv";  
  //Input: All EV data matched into signle file
  private static final String ADDRESS_FILE_EV="EV_full.csv";
  //Input: Generator_Bus_Correspondence
  private static final String ADDRESS_FILE_GenBusCorresp="Generator_Bus_Corresondence_full.csv";
  //Input: expected capacity at busses
  private static final String ADDRESS_FILE_SumByBus="sum_bybus_full.csv";
  //Input: EV coordinates of EIA plants
  private static final String ADDRESS_FILE_EiaEvPlantMatches="EV_coordinates_of_EIA_plants.csv";
  
  public static void main(String[] args) throws IOException {
    
    //these read the input files
    CsvReader reader_eia = new CsvReader(new FileReader(ADDRESS_FILE_EIA));
    CsvReader reader_ev = new CsvReader(new FileReader(ADDRESS_FILE_EV));
    CsvReader reader_GenBusCorresp = new CsvReader(new FileReader(ADDRESS_FILE_GenBusCorresp));
    CsvReader reader_SumByBus = new CsvReader(new FileReader(ADDRESS_FILE_SumByBus));
    CsvReader reader_EV_coordinates_of_EIA_plants = new CsvReader(new FileReader(ADDRESS_FILE_EiaEvPlantMatches));
    
    //initiate variables
    String [] nextLine_eia = {};
    String [] nextLine_ev = {};
    String [] nextLine_GenBusCorresp = {};
    String [] nextLine_SumByBus = {};
    String [] nextLine_EV_coordinates_of_EIA_plants = {};
    Double code_progress = 0.0;
    List <String[]> records_ev = new ArrayList<String[]>();
    List <String[]> records_GenBusCorresp = new ArrayList<String[]>();
    List <String[]> records_BusInfo = new ArrayList<String[]>();
    List <String[]> Unmatched_EIA_units = new ArrayList<String[]>();
    List <String[]> unmatched_EIA_units_2 = new ArrayList<String[]>();
    List <String[]> records_EV_coordinates_of_EIA_plants = new ArrayList<String[]>();
    List <String[]> records_savedForLastResort = new ArrayList<String[]>();
    
    //double array to store current total capacities
    Double [] buscap = new Double[9000000];
    //double array to store EV total capacities and latitudes/longitudes
    Double [] evGenMW = new Double[9000000];
    Double [] latitude = new Double[9000000];
    Double [] longitude = new Double[9000000];
    Double [] evMaxMW = new Double[9000000];
    
    //output file name
    CsvWriter writer_dataset = new CsvWriter("EIA-EV_Gens_1.csv");
    CsvWriter writer_busInfo = new CsvWriter("bussesResult_full.csv");
    CsvWriter writer_matches = new CsvWriter("matches_info.csv");
    writer_dataset.setDelimiter(',');
    
    //skip first lines (titles)
    reader_eia.readRecord();
    reader_ev.readRecord();
    reader_GenBusCorresp.readRecord();reader_GenBusCorresp.readRecord();
    reader_SumByBus.readRecord();
    reader_EV_coordinates_of_EIA_plants.readRecord();
    
    //write titles (first lines)
    String [] title_start = {"newbus","newid"};
    String [] title_blank = {};
    String [] title_eia = reader_eia.getValues();
    String [] title_ev = reader_ev.getValues();
    String [] title_end = {"nox", "so2", "sum", "mwh", "hr", "emissions_nox", "emission_so2", "dollarpermwh"};
    String [] title_matches = {"bus","reduced bus","EIA code","Unit Code","Capacity","distance","match type",
      "GenCap","genMW","MaxMW","lat_eia","long_eia","lat_bus","long_bus"};
    writer_dataset.writeRecord(concat(title_start,concat(title_eia,concat(title_blank,concat(title_ev,title_end)))));
    writer_matches.writeRecord(title_matches);
    
    //store bus info data in array
    while (reader_GenBusCorresp.readRecord()){
      nextLine_GenBusCorresp = reader_GenBusCorresp.getValues(); 
      records_GenBusCorresp.add(nextLine_GenBusCorresp);
    }
    System.out.println("Bus correspondance data is copied");
    
    //store expected capacity data
    while (reader_SumByBus.readRecord()){
      nextLine_SumByBus = reader_SumByBus.getValues(); 
      records_BusInfo.add(nextLine_SumByBus);
      evGenMW[Integer.parseInt(nextLine_SumByBus[0])]=Double.parseDouble(nextLine_SumByBus[1]);
      latitude[Integer.parseInt(nextLine_SumByBus[0])]=Double.parseDouble(nextLine_SumByBus[2]);
      longitude[Integer.parseInt(nextLine_SumByBus[0])]=Double.parseDouble(nextLine_SumByBus[3]);
      evMaxMW[Integer.parseInt(nextLine_SumByBus[0])]=Double.parseDouble(nextLine_SumByBus[4]);
    } 
    System.out.println("Expected capacity data is copied");
    
    //store EV data
    while (reader_ev.readRecord()){
      nextLine_ev = reader_ev.getValues(); 
      records_ev.add(nextLine_ev);
    }
    System.out.println("EV data is copied");
    
    //store match data
    while (reader_EV_coordinates_of_EIA_plants.readRecord()){
      nextLine_EV_coordinates_of_EIA_plants = reader_EV_coordinates_of_EIA_plants.getValues(); 
      records_EV_coordinates_of_EIA_plants.add(nextLine_EV_coordinates_of_EIA_plants);
    }
    System.out.println("match data is copied");
    
    //start looping through EIA units
    while (reader_eia.readRecord()){
      //values from current EIA unit
      nextLine_eia = reader_eia.getValues(); 
      
      //variables
      Boolean found_reducedBus = false;
      Boolean unit_written_to_file = false;
      int i = 0; int k = 0;
      String [] newbus = new String[1];
      String [] newid = new String[1];
      
      //keep track of code progress
      if(code_progress%10 == 0){
        System.out.println("EIA data is " + code_progress/180.50 + "% matched");
      }
      
      //TRY EXACT MATCH (EV provided plant code and generator ID)
      String first_interconection = "none";
      while(!first_interconection.equals("both found")&&i<records_ev.size()){
        if(records_ev.get(i)[3].equals(nextLine_eia[2])&&records_ev.get(i)[4].equals(nextLine_eia[6])&&
           !records_ev.get(i)[342].equals(first_interconection)){
          int a = 0; k = 0;
          
          //remove blanks from bus number
          String s = records_ev.get(i)[1];
          s.replaceAll(" ", "");
          
          //find reduced bus number
          while(!found_reducedBus&&a<records_GenBusCorresp.size()){
            a++;
            if(records_ev.get(i)[0].equals(records_GenBusCorresp.get(k)[0])&&s.equals(records_GenBusCorresp.get(k)[2])){
              newbus[0] = records_GenBusCorresp.get(k)[5];
              newid[0] = records_GenBusCorresp.get(k)[6];
              found_reducedBus=true;
            }
            k++;
          }     
          if(newbus[0]==null){
            Unmatched_EIA_units.add(nextLine_eia); 
            unit_written_to_file = true;
          }
          
          //write matched unit to file and update variables
          if(newbus[0]!=null){
            if(first_interconection.equals("none")){
              writer_dataset.writeRecord(concat(newbus,concat(newid,concat(nextLine_eia,records_ev.get(i)))));
              first_interconection = records_ev.get(i)[342];
            }else{
              writer_dataset.writeRecord(concat(newbus,concat(newid,concat(nextLine_eia,records_ev.get(i)))));
              first_interconection = "both found";
            }
            code_progress++; 
            unit_written_to_file = true;
            Integer bus_number_chosen = Integer.parseInt(records_ev.get(i)[0]);
            
            //update bus capacity array (stores total current capacity at each array)         
            if(buscap[bus_number_chosen]==null){
              buscap[bus_number_chosen]=0.0; 
            }
            if(!(nextLine_eia[18].equals("N")||nextLine_eia[8].equals("OS"))){
              buscap[bus_number_chosen]=buscap[bus_number_chosen]+Double.parseDouble(nextLine_eia[9]);
            }
          }
        }
        i++;
      }
      //if no perfect match was found store unit to an array for later processing
      if(unit_written_to_file==false){
        Unmatched_EIA_units.add(nextLine_eia); 
      }   
      writer_dataset.flush();
    }
    
    //go through array with the remainder of units
    
    //WRITE CANADIAN EV UNITS (see documentation for criteria to be a canadian unit)
    //Update, also add units from mexico (code BJ)
    int u = 0; String [] blank = new String[136]; blank[0]="EV unit";
    while (u<records_ev.size()){
      if(records_ev.get(u)[8].equals("AB")||records_ev.get(u)[8].equals("BC")||records_ev.get(u)[8].equals("BJ")
           ||records_ev.get(u)[8].equals("MB")||records_ev.get(u)[8].equals("NB")||records_ev.get(u)[8].equals("NF")
           ||records_ev.get(u)[8].equals("NL")||records_ev.get(u)[8].equals("NT")||records_ev.get(u)[8].equals("NS")
           ||records_ev.get(u)[8].equals("NU")||records_ev.get(u)[8].equals("ON")||records_ev.get(u)[8].equals("PE")
           ||records_ev.get(u)[8].equals("QC")||records_ev.get(u)[8].equals("SK")||records_ev.get(u)[8].equals("YT")
           ||records_ev.get(u)[8].equals("PQ")||records_ev.get(u)[17].equals("Canada")){
        int b=0; Integer bus = 0;
        
        //find reduced bus number
        while(b<records_GenBusCorresp.size()){
          if(Double.parseDouble(records_ev.get(u)[0])==(Double.parseDouble(records_GenBusCorresp.get(b)[0]))){
            bus = Integer.parseInt(records_GenBusCorresp.get(b)[5]);
          }
          b++; 
        }
        blank[0]=bus+"";
        
        if(bus>0){
          try{
            if(Double.parseDouble(records_ev.get(u)[183])>0){
              //write unit to file          
              writer_dataset.writeRecord(concat(blank,records_ev.get(u)));
              
              //update bus capacity array
              Integer bus_number_chosen = Integer.parseInt(records_ev.get(u)[0]);
              if(buscap[bus_number_chosen]==null){
                buscap[bus_number_chosen]=0.0; 
              }
              buscap[bus_number_chosen]=buscap[bus_number_chosen]+Double.parseDouble(records_ev.get(u)[183]);
            }
          }catch(NumberFormatException e){
            try{
              if(Double.parseDouble(records_ev.get(u)[86])>0){
                //write unit to file          
                writer_dataset.writeRecord(concat(blank,records_ev.get(u)));
                
                //update bus capacity array
                Integer bus_number_chosen = Integer.parseInt(records_ev.get(u)[0]);
                if(buscap[bus_number_chosen]==null){
                  buscap[bus_number_chosen]=0.0; 
                }
                buscap[bus_number_chosen]=buscap[bus_number_chosen]+Double.parseDouble(records_ev.get(u)[86]);
              }
            }catch(NumberFormatException t){};
          };
        }
      }
      u++; 
    }
    
    //WRITE REMAINING EIA UNITS
    
    // FIRST MATCH UNITS WITH PARTIAL EV INFO, THEN WITH NO EV INFO
    int v = 0; 
    while (v<Unmatched_EIA_units.size()){
      
      //keep track of code progress
      if(code_progress%10==0){
        System.out.println("EIA data is " +code_progress/180.50+"% matched");
      }

      // But first, if EIA plant had a match in EV dataset, search around that EV unit coordinates and place at any bus within 1km
      String function_used = "0"; String [] to_write = null; int z = 0; 
      while(z < records_EV_coordinates_of_EIA_plants.size()){
        if(records_EV_coordinates_of_EIA_plants.get(z)[0].equals(Unmatched_EIA_units.get(v)[2])){
          // update coordinates (use ev coordinates)
          Unmatched_EIA_units.get(v)[87]=records_EV_coordinates_of_EIA_plants.get(z)[2];
          Unmatched_EIA_units.get(v)[88]=records_EV_coordinates_of_EIA_plants.get(z)[1];
          // if match is found use the ev coordinates instead of EIA coordiates. Search in 1 km radius. match even if positive need
          to_write = find_neediest(Unmatched_EIA_units.get(v),1,false,records_BusInfo,buscap,evGenMW,evMaxMW,records_GenBusCorresp,true);
          // exit while loop
          z=99999999;
        }
        z++;
      }
   
      // Write result to file
      String [] matches_info = {"na","na","na","na","na","na","na","na","na","na","na","na","na","na"};
      
      if(to_write==null){
        unmatched_EIA_units_2.add(Unmatched_EIA_units.get(v));
        
        matches_info[2] = Unmatched_EIA_units.get(v)[2];
        matches_info[3] = Unmatched_EIA_units.get(v)[6];
        matches_info[4] = Unmatched_EIA_units.get(v)[9];
        matches_info[10] = Unmatched_EIA_units.get(v)[87];
        matches_info[11] = Unmatched_EIA_units.get(v)[88];
        writer_matches.writeRecord(matches_info);
        writer_matches.flush();
      }
      else{
        code_progress++;
        writer_dataset.writeRecord(to_write);
        writer_dataset.flush();
          
        matches_info[0] = to_write[1].substring(5);
        matches_info[1] = to_write[0];
        matches_info[2] = Unmatched_EIA_units.get(v)[2];
        matches_info[3] = Unmatched_EIA_units.get(v)[6];
        matches_info[4] = Unmatched_EIA_units.get(v)[9];
        matches_info[6] = function_used;
        matches_info[7] = buscap[Integer.parseInt(matches_info[0])]+"";
        matches_info[8] = evGenMW[Integer.parseInt(matches_info[0])]+"";
        matches_info[9] = evMaxMW[Integer.parseInt(matches_info[0])]+"";
        matches_info[10] = Unmatched_EIA_units.get(v)[87];
        matches_info[11] = Unmatched_EIA_units.get(v)[88];
        matches_info[12] = latitude[Integer.parseInt(matches_info[0])]+"";
        matches_info[13] = longitude[Integer.parseInt(matches_info[0])]+"";
        matches_info[5] = dist(Double.parseDouble(matches_info[10]),Double.parseDouble(matches_info[12]),
                               Double.parseDouble(matches_info[11]),Double.parseDouble(matches_info[13]))+"";
        writer_matches.writeRecord(matches_info);
        writer_matches.flush();
      }
      v++; 
    }
    
    // OTHER UNITS: NO EV INFO
    v = 0; 
    while (v<unmatched_EIA_units_2.size()){
      
      //keep track of code progress
      if(code_progress%10==0){
        System.out.println("EIA data is " +code_progress/180.50+"% matched");
      }
      
      String [] to_write = null;
      
      
      // Look for needest bus within X km, only if the need is positive (X=3,15,35,60,100)
      // Code stops when a bus is found (because to_write becomes not null)
      // Positive need is current bus capacity < minimum desired capacity (also called expected capacity and equal to the EV PG)
      // Bus matched only if capacity with new unit doesn't exeed EV capacity by more than 5%
      // EV capacity is maximum of 1.05*(EV PG) and 1.05*(EV Max MW)
      // If no qualifying match within 100km, place at neediest bus within larger radius (must have positive need)

      to_write=find_neediest(unmatched_EIA_units_2.get(v),3,true,records_BusInfo,buscap,evGenMW,
                                                  evMaxMW,records_GenBusCorresp,false);
      String function_used = "1";
      if(to_write==null){to_write=find_neediest(unmatched_EIA_units_2.get(v),15,true,records_BusInfo,buscap,evGenMW,
                                              evMaxMW,records_GenBusCorresp,false);
      function_used = "2";
      };
      if(to_write==null){to_write=find_neediest(unmatched_EIA_units_2.get(v),35,true,records_BusInfo,buscap,evGenMW,
                                              evMaxMW,records_GenBusCorresp,false);
      function_used = "3";
      };  
      if(to_write==null){to_write=find_neediest(unmatched_EIA_units_2.get(v),60,true,records_BusInfo,buscap,evGenMW,
                                              evMaxMW,records_GenBusCorresp,false);
      function_used = "4";
      }; 
      if(to_write==null){to_write=find_neediest(unmatched_EIA_units_2.get(v),100,true,records_BusInfo,buscap,evGenMW,
                                              evMaxMW,records_GenBusCorresp,false);
      function_used = "5a";
      }; 
      if(to_write==null){to_write=find_neediest(unmatched_EIA_units_2.get(v),100,true,records_BusInfo,buscap,evGenMW,
                                              evMaxMW,records_GenBusCorresp,true);
      function_used = "5b";
      }; 
      // Same as above, but no restrictions on expected capacityor positive need
      if(to_write==null){to_write=find_neediest(unmatched_EIA_units_2.get(v),3,false,records_BusInfo,buscap,evGenMW,
                                              evMaxMW,records_GenBusCorresp,true);
      function_used = "6";
      };
      if(to_write==null){to_write=find_neediest(unmatched_EIA_units_2.get(v),15,false,records_BusInfo,buscap,evGenMW,
                                              evMaxMW,records_GenBusCorresp,true);
      function_used = "7";
      };
      if(to_write==null){to_write=find_neediest(unmatched_EIA_units_2.get(v),35,false,records_BusInfo,buscap,evGenMW,
                                              evMaxMW,records_GenBusCorresp,true);
      function_used = "8";
      };
      if(to_write==null){to_write=find_neediest(unmatched_EIA_units_2.get(v),60,false,records_BusInfo,buscap,evGenMW,
                                              evMaxMW,records_GenBusCorresp,true);
      function_used = "9";
      };
      if(to_write==null){to_write=find_neediest(unmatched_EIA_units_2.get(v),100,false,records_BusInfo,buscap,evGenMW,
                                              evMaxMW,records_GenBusCorresp,true);
      function_used = "10";
      }; 
      
      // Write result to file
      String [] matches_info = {"na","na","na","na","na","na","na","na","na","na","na","na","na","na"};
      
      if(to_write==null){
        records_savedForLastResort.add(unmatched_EIA_units_2.get(v));
        
        matches_info[2] = unmatched_EIA_units_2.get(v)[2];
        matches_info[3] = unmatched_EIA_units_2.get(v)[6];
        matches_info[4] = unmatched_EIA_units_2.get(v)[9];
        matches_info[10] = unmatched_EIA_units_2.get(v)[87];
        matches_info[11] = unmatched_EIA_units_2.get(v)[88];
        writer_matches.writeRecord(matches_info);
        writer_matches.flush();
      }
      else{
        code_progress++;
        writer_dataset.writeRecord(to_write);
        writer_dataset.flush();
          
        matches_info[0] = to_write[1].substring(5);
        matches_info[1] = to_write[0];
        matches_info[2] = unmatched_EIA_units_2.get(v)[2];
        matches_info[3] = unmatched_EIA_units_2.get(v)[6];
        matches_info[4] = unmatched_EIA_units_2.get(v)[9];
        matches_info[6] = function_used;
        matches_info[7] = buscap[Integer.parseInt(matches_info[0])]+"";
        matches_info[8] = evGenMW[Integer.parseInt(matches_info[0])]+"";
        matches_info[9] = evMaxMW[Integer.parseInt(matches_info[0])]+"";
        matches_info[10] = unmatched_EIA_units_2.get(v)[87];
        matches_info[11] = unmatched_EIA_units_2.get(v)[88];
        matches_info[12] = latitude[Integer.parseInt(matches_info[0])]+"";
        matches_info[13] = longitude[Integer.parseInt(matches_info[0])]+"";
        matches_info[5] = dist(Double.parseDouble(matches_info[10]),Double.parseDouble(matches_info[12]),
                               Double.parseDouble(matches_info[11]),Double.parseDouble(matches_info[13]))+"";
        writer_matches.writeRecord(matches_info);
        writer_matches.flush();
      }
      v++; 
    }
    
    //Last resort placement for units
    int w = 0; 
    while (w<records_savedForLastResort.size()){
      //keep track of code progress
      code_progress++;
      System.out.println("EIA data is " +code_progress/180.50+"% matched");
      
      // compute and assign to neediest bus within larger radius
      String [] to_write = find_neediest(records_savedForLastResort.get(w),250,true,records_BusInfo,
                                                            buscap,evGenMW,evMaxMW,records_GenBusCorresp,false);
      if(to_write==null){to_write = find_neediest(records_savedForLastResort.get(w),500,true,records_BusInfo,
                                                            buscap,evGenMW,evMaxMW,records_GenBusCorresp,false);};
      if(to_write==null){to_write = find_neediest(records_savedForLastResort.get(w),1000,true,records_BusInfo,
                                                            buscap,evGenMW,evMaxMW,records_GenBusCorresp,false);};
      if(to_write==null){to_write = find_neediest(records_savedForLastResort.get(w),4000,false,records_BusInfo,
                                                            buscap,evGenMW,evMaxMW,records_GenBusCorresp,true);};
      writer_dataset.writeRecord(to_write);
      w++;
    }
    
    //this piece of code is optional
    //it enables to create phantom generators at busses that were not fully matched to capacity.
    u = 1; String [] info = new String[6]; 
    String [] title_busInfo = {"bus","capacity","genMW","MaxMW","lat","long"};
    writer_busInfo.writeRecord(title_busInfo);
    while (u<8999999){
      if(buscap[u]==null){
        buscap[u]=0.0; 
      }
      if(evGenMW[u]==null){
        evGenMW[u]=0.0; 
      }
      if(evMaxMW[u]==null){
        evMaxMW[u]=0.0; 
      }
      
      // output information about buses
      info[0] = u+"" ; 
      info[1] = buscap[u]+"" ; 
      info[2] = evGenMW[u]+"" ;
      info[3] = evMaxMW[u]+"" ;
      info[4] = latitude[u]+"" ;
      info[5] = longitude[u]+"" ;
      if(Double.parseDouble(info[1])!=0.0||Double.parseDouble(info[2])!=0.0||Double.parseDouble(info[3])!=0.0){
         writer_busInfo.writeRecord(info);
      }
      
      if(evGenMW[u]>buscap[u]){
        int b=0; Integer bus = 0;
        while(b<records_GenBusCorresp.size()){
          if(u==(Double.parseDouble(records_GenBusCorresp.get(b)[0]))){
            bus = Integer.parseInt(records_GenBusCorresp.get(b)[5]);
          }
          b++; 
        }
        blank = new String[150];
        blank[0]=bus+"";
        blank[1]="bus: "+u;//"phantomGenerator:"+u;
        blank[11]="5000";//(Math.max(evGenMW[u]-buscap[u],0))+"";
        blank[96]="N/A";
        blank[10]="OP";
        
        //if we want phantom generation uncomment line below
        //writer_dataset.writeRecord(blank);
      }
      u++;
      
    }
    writer_busInfo.flush();
    writer_busInfo.close();
    writer_dataset.close();
    System.out.println("Done" );
  }
  
  //function to concatenate string arrays
  public static final String[] concat(String[] s1, String[] s2) {
    String[] combined = new String[s1.length + s2.length];
    System.arraycopy(s1, 0, combined, 0, s1.length);
    System.arraycopy(s2, 0, combined, s1.length, s2.length);
    return combined;
  }
  
  //function to calculate distance in KM between 2 coordinates
  public static final double dist(double lat1, double lat2, double lon1, double lon2) {
    int r = 6371; // km
    double rad = 0.0174532925;
    double dis = Math.acos(Math.sin(lat1*rad)*Math.sin(lat2*rad) + 
                           Math.cos(lat1*rad)*Math.cos(lat2*rad) *
                           Math.cos(lon2*rad-lon1*rad)) * r;
    
    return dis;
  }
  
  // Function to find neediest bus within X km, subjected to specified expected capacity limits
  // Matched bus is returned if needy bus is found within X km that matches specified capacity limits
  // Else null is returned
  public static final String[] find_neediest(String[] info, double search_distance, Boolean only_positive_need, 
                                           List <String[]> records_BusInfo, Double [] buscap, Double [] evGenMW, 
                                           Double [] evMaxMW, List <String[]> records_GenBusCorresp, Boolean mustPlace) {
    
    // Variables
    List <String[]> candidates_for_neediest = new ArrayList<String[]>();
    int b =0;
    Integer current_bus = 0;
    Double current_lat = 0.0;
    Double current_long = 0.0;
    
    // loops through list of all buses
    while (b<records_BusInfo.size()){
      current_lat = Double.parseDouble(records_BusInfo.get(b)[3]);
      current_long = Double.parseDouble(records_BusInfo.get(b)[2]);
      current_bus = Integer.parseInt(records_BusInfo.get(b)[0]);
      
      // Consider bus if within X km of unit
      if(dist(Double.parseDouble(info[87]),current_lat,Double.parseDouble(info[88]),current_long)<search_distance){
        if(buscap[current_bus]==null){
          buscap[current_bus]=0.0; 
        }
        if(evGenMW[current_bus]==null){
          evGenMW[current_bus]=0.0; 
        }
        
        // Calculate capacity statistics
        Double need = evGenMW[current_bus]-buscap[current_bus];
        Double excess = buscap[current_bus]-evMaxMW[current_bus]*1.05;
        Double alt_excess = buscap[current_bus]-evGenMW[current_bus]*1.05;
        
        // Altexcess is a better measure of excess at buses where EVgenMW exceeds EvmaxMW
        // so we use the larger of the two as our measure of excess.
        Double max_excess = Math.max(excess,alt_excess);
        
        String[] s1 = {current_bus+"",need+""};
        
        // if positive need is required
        if(only_positive_need){
          // then must have positive need
          if(need>0){
            // if unit must be placed
            if(mustPlace){
              // place unit
              candidates_for_neediest.add(s1);
            }else 
              // else disqualify placement if would make capacity exceed EV capacity by >5%
              if(max_excess + Double.parseDouble(info[9]) < 0){
              candidates_for_neediest.add(s1);
            }
          }
        }else{
          // no positive need required
          if(mustPlace){
            // if unit must be placed, place unit
            candidates_for_neediest.add(s1);
          }else 
            // else disqualify placement if would make capacity exceed EV capacity by >5%
            if(max_excess + Double.parseDouble(info[9]) < 0){
            candidates_for_neediest.add(s1);
          }
        }
      }
      b++;
    }
    
    String [] blank1={"",""};
    if(candidates_for_neediest.size()>0){
      //find neediest bus among buses found
      Double max = -10000.0;
      double bus2 =0.0;   
      for(int n =0; n<candidates_for_neediest.size();n++){
        if(Double.parseDouble(candidates_for_neediest.get(n)[1])>max){
          max =  Double.parseDouble(candidates_for_neediest.get(n)[1]);
          bus2 = Double.parseDouble(candidates_for_neediest.get(n)[0]);
        }
      }
      
      //find reduced bus of neediest bus
      int d=0; Integer bus = 0;
      while(d<records_GenBusCorresp.size()){
        if(bus2==(Double.parseDouble(records_GenBusCorresp.get(d)[0]))){
          bus = Integer.parseInt(records_GenBusCorresp.get(d)[5]);
        }
        d++; 
      }
      
      int bus2int = (int)bus2;
      blank1[0]=bus+"";
      blank1[1]="bus: " + bus2int;
      
      //update bus capacity array
      Integer bus_number_chosen = Integer.parseInt(bus2int+"");
      if(buscap[bus_number_chosen]==null){
        buscap[bus_number_chosen]=0.0; 
      }
      if(!(info[18].equals("N")||info[8].equals("OS"))){
        if(info[10].equals("")){
          buscap[bus_number_chosen]=buscap[bus_number_chosen]+Double.parseDouble(info[9]);
        }else{
          buscap[bus_number_chosen]=buscap[bus_number_chosen]+Double.parseDouble(info[10]);
        }
      }
    }
    
    //return record with reduced bus number
    if(candidates_for_neediest.size()>0){
      return concat(blank1,info);
    }else{
      return null;
    }
  }
}

