//**********************************************************
// Program: add_emissions_EV.java
//
// Author: Charles Marquet
//
// Description: This program adds emission rates to EV firstrate files    
//
// Input: - The three firstrate files
//        - The EIA-EV_Gens.csv file 
//        - The emissions matrix file
// 
// Output: -Three firstrate files, with SO2 and SOx emission rates
//
// Need the free csv reader/writer from: http://sourceforge.net/projects/javacsv/
//*********************************************************

import java.io.*;
import java.lang.String;
import com.csvreader.CsvReader;
import com.csvreader.CsvWriter;

public class add_emissions_EV {
  //these are the input file names
  private static final String ADDRESS_FILE="EIA-EV_Gens.csv";
  private static final String ADDRESS_FILE1="wecc.csv";
  private static final String ADDRESS_FILE2="ercot.csv";
  private static final String ADDRESS_FILE3="mmwg.csv";
  private static final String ADDRESS_FILE4="emissions_matrix.csv";
  
  public static void main(String[] args) throws IOException {
    
    //these read the input files
    CsvReader reader = new CsvReader(new FileReader(ADDRESS_FILE));
    CsvReader reader1 = new CsvReader(new FileReader(ADDRESS_FILE1));
    CsvReader reader2 = new CsvReader(new FileReader(ADDRESS_FILE2));
    CsvReader reader3 = new CsvReader(new FileReader(ADDRESS_FILE3));
    CsvReader reader4 = new CsvReader(new FileReader(ADDRESS_FILE3));
    
    //initiate variables
    String [] nextLine;
    String [] nextLine1;
    String [] nextLine2;
    String [] nextLine3;
    String [] nextLine4;
    
    //output file names
    CsvWriter writer1 = new CsvWriter("wecc_emissions.csv");
    CsvWriter writer2 = new CsvWriter("ercot_emissions.csv");
    CsvWriter writer3 = new CsvWriter("mmwg_emissions.csv");
    CsvWriter writer4 = new CsvWriter("EIA-EV_Gens_modforEV.csv");
    writer1.setDelimiter(',');
    writer2.setDelimiter(',');
    writer3.setDelimiter(',');
    writer4.setDelimiter(',');
    
    //Enter average emissions rates in the emissions matrix file
    //It will be used if matching EIA-EPA unit cannot be found
    //Avarages are by combination of prime mover & energy source
    
    //write titles of output files
    //first title row
    reader1.readRecord();
    reader2.readRecord();
    reader3.readRecord();
    nextLine1 = reader1.getValues();
    nextLine2 = reader2.getValues();
    nextLine3 = reader3.getValues();
    writer1.writeRecord(nextLine1);
    writer2.writeRecord(nextLine2);
    writer3.writeRecord(nextLine3);
    
    //second title row
    reader1.readRecord();
    reader2.readRecord();
    reader3.readRecord();
    nextLine1 = reader1.getValues();
    nextLine2 = reader2.getValues();
    nextLine3 = reader3.getValues();
    String [] title = {"SO2 Rate","NOx Rate"};
    
    //add additional column names (eia & epa)
    reader.readRecord();
    nextLine = reader.getValues(); 
    String [] title_to_add=new String[120];
    for(int h = 0; h<32 ; h++){
      title_to_add[h]=nextLine[2+h];
    }
    for(int h = 0; h<52 ; h++){
      title_to_add[32+h]=nextLine[37+h];
    }
    for(int h = 0; h<35 ; h++){
      title_to_add[84+h]=nextLine[91+h];
    }
    title_to_add[119]=nextLine[483];
    
    writer1.writeRecord(concat(nextLine1,concat(title,title_to_add))); 
    writer2.writeRecord(concat(nextLine2,concat(title,title_to_add))); 
    writer3.writeRecord(concat(nextLine3,concat(title,title_to_add))); 
    writer1.flush();
    writer2.flush();
    writer3.flush();
    
    //make modified version of EIA-EPA file (change columns order)
    String [] modifed_eiaepa_data = {};
    while (reader.readRecord()){
      nextLine = reader.getValues(); 
      if(!nextLine[136].equals("")&&!nextLine[3].equals("")){
        modifed_eiaepa_data = new String[]{nextLine[4],nextLine[8],nextLine[480],nextLine[479]};
        
        String [] eia_to_add=new String[120];
        for(int h = 0; h<32 ; h++){
          eia_to_add[h]=nextLine[2+h];
        }
        for(int h = 0; h<52 ; h++){
          eia_to_add[32+h]=nextLine[37+h];
        }
        for(int h = 0; h<35 ; h++){
          eia_to_add[84+h]=nextLine[91+h];
        }
        eia_to_add[119]=nextLine[483];
        
        writer4.writeRecord(concat(modifed_eiaepa_data,eia_to_add));
      }
    }
    writer4.flush();
    writer4.close();
    
    System.out.println("Setup done" );
    //add emissions data
    
    //variables
    String [] emissions_to_add ={};
    String [] eiaepa_to_add ={};
    Boolean emissions_found = false;
    
    //add emissions to WECC
    while (reader1.readRecord()){
      //get values from the current row
      nextLine1 = reader1.getValues(); 
      emissions_found = false;
      
      //load EIA-EPA file, skip title row
      reader = new CsvReader(new FileReader("EIA-EV_Gens_modforEV.csv"));
      emissions_to_add=new String[]{"0","0"};
      eiaepa_to_add=new String[120];
      
      //see if a matched EIA-EPA unit exists, if so use its emissions rate
      while (reader.readRecord()&&!emissions_found){
        nextLine = reader.getValues(); 
        if(nextLine1[3].equals(nextLine[0])&&nextLine1[4].equals(nextLine[1])){
          emissions_to_add=new String[]{nextLine[2],nextLine[3]};
          
          //add additional columns (eia & epa)
          for(int q = 0; q<120 ; q++){
            eiaepa_to_add[q]=nextLine[4+q];
          }
          
          emissions_found = true;
        }
      }
      //else use an estimated emissions rate
      if(!emissions_found){
        reader4 = new CsvReader(new FileReader(ADDRESS_FILE4));
        reader4.readRecord();
        while (reader4.readRecord()&&!emissions_found){
          nextLine4 = reader4.getValues(); 
          if(nextLine1[17].equals(nextLine4[0])&&nextLine1[18].equals(nextLine4[1])){
            emissions_to_add=new String[]{nextLine4[2],nextLine4[3]};
            emissions_found = true;
          }
        }
      }
      //ensure hydro units have no emissions
      if(nextLine1[17].equals("Hydro")){
        emissions_to_add=new String[]{"0","0"};
      }
      writer1.writeRecord(concat(nextLine1,concat(emissions_to_add,eiaepa_to_add)));
    }
    
    writer1.flush();
    writer1.close();
    System.out.println("WECC done" );
    
    //add emissions to ERCOT
    while (reader2.readRecord()){
      //get values from the current row
      nextLine2 = reader2.getValues(); 
      emissions_found = false;
      
      //load EIA-EPA file, skip title row
      reader = new CsvReader(new FileReader("EIA-EV_Gens_modforEV.csv"));
      emissions_to_add=new String[]{"0","0"};
      eiaepa_to_add=new String[120];
      
      //see if a matched EIA-EPA unit exists, if so use its emissions rate
      while (reader.readRecord()&&!emissions_found){
        nextLine = reader.getValues(); 
        if(nextLine2[3].equals(nextLine[0])&&nextLine2[4].equals(nextLine[1])){
          emissions_to_add=new String[]{nextLine[2],nextLine[3]};
          
          //add additional columns (eia & epa)
          for(int q = 0; q<120 ; q++){
            eiaepa_to_add[q]=nextLine[4+q];
          }
          
          emissions_found = true;
        }
      }
      //else use an estimated emissions rate
      if(!emissions_found){
        reader4 = new CsvReader(new FileReader(ADDRESS_FILE4));
        reader4.readRecord();
        while (reader4.readRecord()&&!emissions_found){
          nextLine4 = reader4.getValues(); 
          if(nextLine2[17].equals(nextLine4[0])&&nextLine2[18].equals(nextLine4[1])){
            emissions_to_add=new String[]{nextLine4[2],nextLine4[3]};
            emissions_found = true;
          }
        }
      }
      //ensure hydro units have no emissions
      if(nextLine2[17].equals("Hydro")){
        emissions_to_add=new String[]{"0","0"};
      }
      writer2.writeRecord(concat(nextLine2,concat(emissions_to_add,eiaepa_to_add)));
    }
    
    writer2.flush();
    writer2.close();
    System.out.println("ERCOT done" );
    
    //add emissions to MMWG
    while (reader3.readRecord()){
      //get values from the current row
      nextLine3 = reader3.getValues(); 
      emissions_found = false;
      
      //load EIA-EPA file, skip title row
      reader = new CsvReader(new FileReader("EIA-EV_Gens_modforEV.csv"));
      emissions_to_add=new String[]{"0","0"};
      eiaepa_to_add=new String[120];
      
      //see if a matched EIA-EPA unit exists, if so use its emissions rate
      while (reader.readRecord()&&!emissions_found){
        nextLine = reader.getValues(); 
        if(nextLine3[3].equals(nextLine[0])&&nextLine3[4].equals(nextLine[1])){
          emissions_to_add=new String[]{nextLine[2],nextLine[3]};
          
          //add additional columns (eia & epa)
          for(int q = 0; q<120 ; q++){
            eiaepa_to_add[q]=nextLine[4+q];
          }
          
          emissions_found = true;
        }
      }
      //else use an estimated emissions rate
      if(!emissions_found){
        reader4 = new CsvReader(new FileReader(ADDRESS_FILE4));
        reader4.readRecord();
        while (reader4.readRecord()&&!emissions_found){
          nextLine4 = reader4.getValues(); 
          if(nextLine3[17].equals(nextLine4[0])&&nextLine3[18].equals(nextLine4[1])){
            emissions_to_add=new String[]{nextLine4[2],nextLine4[3]};
            emissions_found = true;
          }
        }
      }
      //ensure hydro units have no emissions
      if(nextLine3[17].equals("Hydro")){
        emissions_to_add=new String[]{"0","0"};
      }
      writer3.writeRecord(concat(nextLine3,concat(emissions_to_add,eiaepa_to_add)));
    }
    
    writer3.flush();
    writer3.close();
    System.out.println("MMWG done" );
    
    //message indicating code is done running
    System.out.println("Emission rate columns complete, check files" );
    
  }
  //function to concatenate string arrays
  public static final String[] concat(String[] s1, String[] s2) {
    String[] erg = new String[s1.length + s2.length];
    System.arraycopy(s1, 0, erg, 0, s1.length);
    System.arraycopy(s2, 0, erg, s1.length, s2.length);
    return erg;
  }
}

