//**********************************************************
// Program: damages.java
//
// Author: Charles Marquet, cmm363@cornell.edu
//
// Description: This program calculates the environmental damages for all generation units.
//              
//
// Input: -Constants (from Abt and latimer Associates)
//
//        -EIA-EPA matched data 
//             Columns A-BK is EIA data
//             Columns BL-CP is stack data
//             Column CQ is EIA fuel category per unit
//             Columns CR-EL is EPA data matched by match.java
//             Column EM is the fips code for each EIA units
//
//        -Population data 
//             US census data by county matched with abt index for counties 1-3080 using file SR_matrix_readme)
//             Errors found in file: county 12025 is 12086, 29193 is 29186
//
//        -Transfer coeficients (from Abt, modified to remove errors)
//
//        -State % of population over 30 (from US census data)
//
// Need the free csv reader/writer from: http://sourceforge.net/projects/javacsv/
//*********************************************************

import java.io.*;
import java.lang.String;
import java.lang.Math;
import java.lang.Integer;
import com.csvreader.CsvReader;
import com.csvreader.CsvWriter;
import java.util.*;


public class damages {
  //these are the input file names
  //Input: EIA-EPA-stacks matched file
  private static final String ADDRESS_FILE="EIA-EV-emissions_Gens.csv";//"match_NE_fips.csv";  
  //Input: Transfer Coeficients 
  private static final String ADDRESS_FILE2="TC.csv";
  //Input: population data by county
  private static final String ADDRESS_FILE3="pop.csv";
  //Input: % of population 30+ years by state
  private static final String ADDRESS_FILE4="state.csv";
  
  public static void main(String[] args) throws IOException {
    
    //these read the input files
    CsvReader reader = new CsvReader(new FileReader(ADDRESS_FILE));
    CsvReader reader2= new CsvReader(new FileReader(ADDRESS_FILE2));
    CsvReader reader3= new CsvReader(new FileReader(ADDRESS_FILE3));
    CsvReader reader4= new CsvReader(new FileReader(ADDRESS_FILE4));
    
    //constants
    Double beta =0.0046257;
    Double y =0.0082;
    Double noxcoef =1.7415;
    Double so2coef =2.065;
    Double life =8360000.0; //in 2013, as measured in 2013 dollars.
    Double secperyear=31536000.0;
    Double mgperton =1000000000000.0;
    Double metrictons =.907186;
    
    //initiate variables
    List <String[]> records = new ArrayList<String[]>();
    String [] nextLine = {};
    String [] nextLine2 = {};
    String [] nextLine3 = {};
    Double progress = 0.0;
    List <String[]> population = new ArrayList<String[]>();
    
    //output file name
    CsvWriter writer = new CsvWriter("EIA-EV_Gens.csv");
    CsvWriter writer2 = new CsvWriter("damages_per_county.csv");
    writer.setDelimiter(',');
    writer2.setDelimiter(',');
    
    //skip first lines (titles)
    reader.readRecord();
    //reader2.readRecord();
    reader3.readRecord();
    reader4.readRecord();
    
    //write titles (first lines)
    String [] title1 = reader.getValues();
    String [] title_damages = {"index","nox_damages_per_pound","so2_damages_per_pound"};
    String [] title2 = {"damages","nox_damages_permwh","so2_damages_permwh"};
    writer.writeRecord(concat(title1,title2));
    writer2.writeRecord(title_damages);
    
    //store pop in array
    while (reader3.readRecord()){
      nextLine3 = reader3.getValues(); 
      population.add(nextLine3);
    }
    
    // For each county/stack height combination, compute the person-ppms per pound of so2 (sumso2) and nox (sumnox)
    // to compute: for each county/stack height combination, sum all 3080 receptor counties times their population
    Double progress2 = 0.0;
    int track = 0;
    Double sumnox = 0.0;
    Double sumso2 = 0.0;
    Double currentpop = 0.0;
    int remainder = 0;
    records = new ArrayList<String[]>();
    // loop through TCs
    while (reader2.readRecord()){
      progress2++; track++;
      nextLine2 = reader2.getValues();
      int a = Integer.parseInt(nextLine2[2]);
      currentpop = Double.parseDouble(population.get(a-1)[1]);
      
      //add to the sum
      try {
        sumnox  = sumnox + currentpop * Double.parseDouble(nextLine2[5]);
        sumso2  = sumso2 + currentpop * Double.parseDouble(nextLine2[4]);
      }catch (ArrayIndexOutOfBoundsException e){};
      
      // record sum to an array if all of a county's TCs*population have been summed
      if(track ==3080){
        remainder++;
        String[] d = {sumso2+"",sumnox+""};
        records.add(d);
        
        // this creates output for damage per county file 
        if(remainder%4==3){
          Double factor = beta * y * life * mgperton / secperyear * metrictons / 2000;
          String[] d2 = {Integer.parseInt(nextLine2[1])+"",factor * noxcoef *  sumso2+"",factor * so2coef * sumnox+""};
          writer2.writeRecord(d2);
        }
        
        sumnox = 0.0;
        sumso2 = 0.0;
        track = 0;
      }
      
      //keep track of progress
      if(progress2%100000==0){
        System.out.println("TC are " +progress2/(4*3080*3080)*100+"% copied");
      }
    }
    writer2.flush();
    writer2.close();
    
    //calculate health damages from emissions for each generator
    //this loops through the EIA-EPA data
    while (reader.readRecord()){
      //keep track of progress
      progress++;
      if(true){//progress%100==0){
        Double percent = progress/126.2;
        percent = (double)Math.round(percent * 10000) / 13796;
        System.out.println(percent+"% done");
      }
      
      // nextLine[] is an array of values from the current line
      //records = new ArrayList<String[]>();
      nextLine = reader.getValues();
      
      //gather useful values from current unit
      Double fips;
      try {fips = Double.parseDouble(nextLine[135]);}catch (NumberFormatException e){fips=0.0;};
      Double mwh1;
      try {mwh1 = Double.parseDouble(nextLine[100]);}catch (NumberFormatException e){mwh1=0.0;};
      Double mwh2; 
      try {mwh2 = Double.parseDouble(nextLine[101])/6.77;}catch (NumberFormatException e){mwh2=0.0;};
      Double capacity;
      try {if(Double.parseDouble(nextLine[11])>0.0){capacity = Double.parseDouble(nextLine[11]);}
        else{capacity = Double.parseDouble(nextLine[11]);};}catch (NumberFormatException e){
          try {capacity=Double.parseDouble(nextLine[9]);}catch (NumberFormatException f){
            capacity=Double.parseDouble(nextLine[319]);};};
        Double esh;
        try {esh = Double.parseDouble(nextLine[35]);}catch (NumberFormatException e){
          try{esh = Double.parseDouble(nextLine[36]);}catch(NumberFormatException f){esh=0.0;};};
        Double age;
        try {age = 2012.0-Double.parseDouble(nextLine[16]);}catch (NumberFormatException e){age=40.94;};
        
        //calculate MWH
        Double mwh = 0.0;
        if(mwh1>0){
          mwh=mwh1;
        }else if(mwh2>0){
          mwh=mwh2;
        }else {
          //estimate MWh if missing
          mwh = capacity *2531;
        }
        
        //calculate stack height category
        int eshcategory = 0;
        nextLine[36]=esh+"";
        if (esh==0.0){
          //based on regression of esh vs capacity
          esh=106.59+capacity*0.586+age*3.149;
          nextLine[36]=esh+"";
        }
        if(esh>500){
          eshcategory = 4;
        }else if(esh>250){
          eshcategory = 3;
        }else {
          eshcategory = 2;
        }
        
        //calculate heat rate
        Double hr =10.4;
        hr = Double.parseDouble(nextLine[483]);
        
        // read through TC file
        reader2 = new CsvReader(new FileReader(ADDRESS_FILE2));
        
        //go through pop file and find county index (1-3080) based on the current unit's fips code
        reader3 = new CsvReader(new FileReader(ADDRESS_FILE3));
        reader3.readRecord();
        Double index = 0.0;
        while (reader3.readRecord()){
          nextLine3 = reader3.getValues(); 
          if(Double.parseDouble(nextLine3[0])==fips){
            index = Double.parseDouble(nextLine3[2]);
          }
        }
        
        Double sum = 0.0; Double sum_nox = 0.0; Double sum_so2 = 0.0;
        
        // get TC sum based on county index (1-3080) and stack height category (1-4)
        Double so2 = 0.0;
        Double nox = 0.0;
        
        if(!(index==0.0)){
          so2 = Double.parseDouble(records.get((int)((index-1)*4+(eshcategory)-1))[0]);
          if(so2==0.0){
            so2 = Double.parseDouble(records.get((int)((index-1)*4+(1)-1))[0]);
          }
          nox = Double.parseDouble(records.get((int)((index-1)*4+(eshcategory-1)))[1]);
          if(nox==0.0){
            nox = Double.parseDouble(records.get((int)((index-1)*4+(1-1)))[1]);
          }
        }
        
        //compute unit's emissions, 
        Double emissionso2 =0.0;
        emissionso2 = Double.parseDouble(nextLine[480])*hr*mwh/2000;
        
        Double emissionnox = 0.0;
        emissionnox = Double.parseDouble(nextLine[479])*hr*mwh/2000;
        
        // intermediate step to estimate damages (from abt associates) 
        // total emissions * person ppm per pound * factors from abt formula
        sum = (so2*so2coef*emissionso2+nox*noxcoef*emissionnox)*.593;
        sum_nox = (nox*noxcoef*emissionnox)*.593;
        sum_so2 = (so2*so2coef*emissionso2)*.593;
        
        // calculate total damages per year for the unit
        Double damages = 0.0; Double damages_nox = 0.0; Double damages_so2 = 0.0;
        damages = beta * y * life * (mgperton / secperyear) * metrictons * sum;
        damages_nox = beta * y * life * (mgperton / secperyear) * metrictons * sum_nox;
        damages_so2 = beta * y * life * (mgperton / secperyear) * metrictons * sum_so2;
        
        //calculate damages per mwh
        Double dollarpermwh = 0.0; Double dollarpermwh_nox = 0.0;Double dollarpermwh_so2 = 0.0;
        dollarpermwh = damages / mwh;
        dollarpermwh_nox = damages_nox / mwh;
        dollarpermwh_so2 = damages_so2 / mwh;
        
        //output columns
        int cols = 487;
        nextLine[cols] = dollarpermwh*mwh + "";
        nextLine[cols-1] = dollarpermwh + "";
        nextLine[cols-2] = emissionso2 + "";
        nextLine[cols-3] = emissionnox + "";
        nextLine[cols-5] = mwh + "";
        nextLine[cols-6] = sum + "";
        String [] additional_records = {"",""};
        additional_records[0] = dollarpermwh_nox+"";
        additional_records[1] = dollarpermwh_so2+"";
        
        //write results
        writer.writeRecord(concat(nextLine,additional_records));
        writer.flush();
    }
    
    writer.flush();
    writer.close();
    
    //print results of match
    System.out.println("Done" );
    
  }
  //function to concatenate string arrays
  public static final String[] concat(String[] s1, String[] s2) {
    String[] erg = new String[s1.length + s2.length];
    System.arraycopy(s1, 0, erg, 0, s1.length);
    System.arraycopy(s2, 0, erg, s1.length, s2.length);
    return erg;
  }
}

