//**********************************************************
// Program: regression_finnish.java
//
// Author: Charles Marquet
//
// Description: This program checks the emissions data and generates columns for matpower input   
//
// Input: -EIA-EPA-EV matched data
//
// Need the free csv reader/writer from: http://sourceforge.net/projects/javacsv/
//*********************************************************

import java.io.*;
import java.lang.String;
import java.lang.Math;
import com.csvreader.CsvReader;
import com.csvreader.CsvWriter;
import java.util.*;


public class generate_columns {
  //these are the input file names
  //Input: EIA-EPA-stacks matched file
  private static final String ADDRESS_FILE="EIA-EV_Gens.csv";
  private static final String ADDRESS_FILE2="interconnections.csv";
  private static final String ADDRESS_FILE_GenBusCorresp="Generator_Bus_Corresondence_full.csv";
  
  public static void main(String[] args) throws IOException {
    
    //these read the input files
    CsvReader reader = new CsvReader(new FileReader(ADDRESS_FILE));
    CsvReader reader2 = new CsvReader(new FileReader(ADDRESS_FILE2));
    CsvReader reader_GenBusCorresp = new CsvReader(new FileReader(ADDRESS_FILE_GenBusCorresp));
    CsvReader reader_canada_search = new CsvReader(new FileReader(ADDRESS_FILE));
    
    //initiate variables
    String [] nextLine;
    String [] nextLine_interconnection_file;
    String [] nextLine_GenBusCorresp = {};
    List <String[]> records_GenBusCorresp = new ArrayList<String[]>();
    Double code_progress = 0.0;
    
    //store bus info data in array
    reader_GenBusCorresp.readRecord();reader_GenBusCorresp.readRecord();
    while (reader_GenBusCorresp.readRecord()){
      nextLine_GenBusCorresp = reader_GenBusCorresp.getValues(); 
      records_GenBusCorresp.add(nextLine_GenBusCorresp);
    }
    System.out.println("Bus correspondance data is copied");
    
    //output file name
    CsvWriter writer = new CsvWriter("EIA-EV_Gens_Ray.csv");
    writer.setDelimiter(',');
    String [] line = new String[29];
    
    //Calculate average EV fuel/VOM prices
    //default values
    Double avgEVgas = 4.955;
    Double avgOMgas = 1.771;
    Double avgEVcoal = 2.557;
    Double avgOMcoal = 1.109;
    Double avgEVoil = 18.448;
    Double avgOMoil = 4.474;
    //sums
    int numbgasEVunits = 0;
    int numbcoalEVunits = 0;
    int numboilEVunits = 0;
    Double total_avgEVgas = 0.0;
    Double total_avgOMgas = 0.0;
    Double total_avgEVcoal = 0.0;
    Double total_avgOMcoal = 0.0;
    Double total_avgEVoil = 0.0;
    Double total_avgOMoil = 0.0;
    
    //load file
    reader = new CsvReader(new FileReader(ADDRESS_FILE));
    reader.readRecord();
    
    //loop through file to calculate averages
    while (reader.readRecord()){
      nextLine = reader.getValues(); 
      if(!nextLine[151].equals("")){
        //calculate fuel type
        line[22]=nextLine[17];
        if(line[22].equals("")){
          if(nextLine[153].equals("Coal")){
            line[22]="coal";
          }else if(nextLine[153].equals("Natural Gas")){
            line[22]="ng";
          }else if(nextLine[153].equals("Distillate Fuel Oil (Diesel,FO1,FO2,FO4)")||nextLine[153].equals("Jet Fuel")||nextLine[153].equals("Residual Fuel Oil (FO5, FO6)")){
            line[22]="oil";
          }else{
            line[22]="other";
          }
        }else{
          if(nextLine[17].equals("ANT")||nextLine[17].equals("BIT")||nextLine[17].equals("LIG")||nextLine[17].equals("RC")||nextLine[17].equals("SUB")||nextLine[17].equals("WC")){
            line[22]="coal";
          }else if(nextLine[17].equals("NG")||nextLine[17].equals("PG")||nextLine[17].equals("SG")||nextLine[17].equals("SGC")){
            line[22]="ng";
          }else if(nextLine[17].equals("DFO")||nextLine[17].equals("RFO")||nextLine[17].equals("JF")||nextLine[17].equals("KER")||nextLine[17].equals("WO")||nextLine[17].equals("PC")){
            line[22]="oil";
          }else{
            line[22]="other";
          }      
        }
        
        if(line[22].equals("ng")){
          numbgasEVunits++;
          total_avgEVgas=total_avgEVgas+Double.parseDouble(nextLine[151]);
          total_avgOMgas=total_avgOMgas+Double.parseDouble(nextLine[152]);
        }
        if(line[22].equals("coal")){
          numbcoalEVunits++;
          total_avgEVcoal=total_avgEVcoal+Double.parseDouble(nextLine[151]);
          total_avgOMcoal=total_avgOMcoal+Double.parseDouble(nextLine[152]);
        }
        if(line[22].equals("oil")){
          numboilEVunits++;
          total_avgEVoil=total_avgEVoil+Double.parseDouble(nextLine[151]);
          total_avgOMoil=total_avgOMoil+Double.parseDouble(nextLine[152]);
        }
      }
    }
    avgEVgas = total_avgEVgas/numbgasEVunits;
    avgOMgas = total_avgOMgas/numbgasEVunits;
    avgEVcoal = total_avgEVcoal/numbcoalEVunits;
    avgOMcoal = total_avgOMcoal/numbcoalEVunits;
    avgEVoil = total_avgEVoil/numboilEVunits;
    avgOMoil = total_avgOMoil/numboilEVunits;
    System.out.println("Averages calculated");
    
    //write title
    reader = new CsvReader(new FileReader(ADDRESS_FILE));
    reader.readRecord();
    nextLine = reader.getValues(); 
    String [] title = nextLine;
    nextLine[158]="Fuel cost per mmbtu";
    nextLine[159]="BaseCost sans gas coal and oil cost";
    nextLine[160]="HR used";
    nextLine[161]="VOM used";
    String [] title1 = {"Interconnection","reduced_bus2","reduced_bus1","original_busnum", "unreduced_bus2", "PG", 
      "QG", "QMAX", "QMIN", "VG", "mBase", "Gen_Status", "PMAX", "PMIN", "Pc1", "Pc2", "Qc1min", "Qc1max", "Qc2min",
      "Qc2max", "ramp_agc", "ramp_10", "ramp_30", "ramp_q", "apf", "Fuel Type", "MODEL", "STARTUP", "SHUTDOWN", "NCOST", 
      "Gen Base Cost", "C0", "CO2", "NOX", "SO2", "Env Cost", "nox_damages_permwh", "so2_damages_permwh",
      "Fuel cost per mmbtu", "HR used","VOM used","BaseCost sans gas coal oil cost","name","Nation","STATEorPROVINCE"};
    title[0]="";
    writer.writeRecord(concat(title1,title)); 
    writer.flush();
    
    //edit emissions data
    reader = new CsvReader(new FileReader(ADDRESS_FILE));
    reader.readRecord();
    
    line = new String[44];
    while (reader.readRecord()){
      nextLine = reader.getValues(); 
      
      //keep track of code progress
      code_progress++;
      if(code_progress%50==0){
        System.out.println("Data is " +code_progress/187.0+"% processed");
      }

      //old reduced bus number
      line[1]=nextLine[0];
      
      //old original bus number
      line[2]=nextLine[136];
      if(line[2].equals("")){try{line[2]=nextLine[1].substring(5);}catch(StringIndexOutOfBoundsException e){line[2]="na";}}
      
      //find new reduced bus information
      Boolean found_reducedBus = false; 
      int k = 0; 
      String [] newbus = new String[2];
      while(!found_reducedBus && k < records_GenBusCorresp.size()){
        if(line[2].equals(records_GenBusCorresp.get(k)[0])){
          newbus[0] = records_GenBusCorresp.get(k)[8];
          newbus[1] = records_GenBusCorresp.get(k)[9];
          found_reducedBus=true;
        }
        k++;
      }  
      
      //new original bus number
      line[3]=newbus[0];
      
      //new reduced bus number
      line[0]=newbus[1];
      
      //find max capacity
      Double cap1 = 0.0;
      Double cap2 = 0.0;
      Double cap3 = 0.0;
      Double cap4 = 0.0;
      Double cap5 = 0.0;
      try{ cap1 = Double.parseDouble(nextLine[12]);} catch(NumberFormatException g){}
      try{ cap2 = Double.parseDouble(nextLine[11]);} catch(NumberFormatException g){}
      try{ cap3 = Double.parseDouble(nextLine[222]);} catch(NumberFormatException g){}
      try{ cap4 = Double.parseDouble(nextLine[319]);} catch(NumberFormatException g){}
      try{ cap5 = Double.parseDouble(nextLine[471]);} catch(NumberFormatException g){}
      String capmax = Math.max(cap1,Math.max(cap2,Math.max(cap3,Math.max(cap4,cap5))))+"";

      //PG
      line[4]=nextLine[471];
      if(line[4].equals("")){line[4]=nextLine[12];}
      if(line[4].equals("")){line[4]=nextLine[11];}
      
      //QG
      line[5]=nextLine[472];
      if(line[5].equals("")){line[5]="0";}
      
      //qmax
      line[6]=nextLine[230];
      if(line[6].equals("")){line[6]=Double.parseDouble(capmax)/2+"";}
      
      //qmin
      if(nextLine[229].equals("")){line[7]=-Double.parseDouble(line[6])+"";}else{line[7]=Double.parseDouble(nextLine[229])+"";}
      
      //Voltage
      line[8]=nextLine[201];
      if(line[8].equals("")){line[8]="1";}
      
      //mBase
      line[9]=nextLine[174];
      if(line[9].equals("")){line[9]="100";}
      
      //Gen status
      line[10]="";
      if(nextLine[20].equals("N")||nextLine[10].equals("OS")){
        line[10]="0";
      }else{
        line[10]="1";
      }
      
      //pmax
      line[11]=capmax;
      
      //pmin
      line[12]="";
      if(!nextLine[227].equals("")&&Double.parseDouble(nextLine[228])>0){
        line[12]=Double.parseDouble(capmax)*Double.parseDouble(nextLine[227])/Double.parseDouble(nextLine[228])+"";
      }else{
        //estimate pmin based on average ratios
        if(nextLine[9].equals("ST")){
          line[12]=Double.parseDouble(capmax)*0.311227217+"";
        }else if(nextLine[9].equals("GT")){
          line[12]=Double.parseDouble(capmax)*0.310937585+"";
        }else if(nextLine[9].equals("CT")){
          line[12]=Double.parseDouble(capmax)*0.241366569+"";
        }else if(nextLine[9].equals("CA")){
          line[12]=Double.parseDouble(capmax)*0.213117487+"";
        }else if(nextLine[9].equals("CS")){
          line[12]=Double.parseDouble(capmax)*0.206956943+"";
        }else if(nextLine[9].equals("HY")){
          line[12]=Double.parseDouble(capmax)*0+"";
        }else if(nextLine[9].equals("PS")){
          line[12]=Double.parseDouble(capmax)*-0.753219531+"";
        }else if(nextLine[9].equals("PV")){
          line[12]=Double.parseDouble(capmax)*0.273609885+"";
        }else if(nextLine[9].equals("IC")){
          line[12]=Double.parseDouble(capmax)*0.065921454+"";
        }else{
          line[12]="0";
        }
      }
      //if bad value replace by 0
      if (line[12].equals("NaN")) {
        line[12]="0.0";
      }
      
      //columns with all 0s
      line[13]="0";
      line[14]="0";
      line[15]="0";
      line[16]="0";
      line[17]="0";
      line[18]="0";
      line[19]="0";
      line[20]="0";
      line[21]="0";
      line[22]="0";
      line[23]="0";
      
      //fuel type    
      line[24]=nextLine[17];
      if(nextLine[17].equals("") || nextLine[17].equals("RFO") || nextLine[17].equals("DFO") || nextLine[17].equals("KER")){
        if(nextLine[153].equals("Coal")){
          line[24]="coal";
        }else if(nextLine[153].equals("Natural Gas")){
          line[24]="ng";
        }else if(nextLine[153].equals("Wind")){
          line[24]="wind";
        }else if(nextLine[153].equals("Nuclear")){
          line[24]="nuclear";
        }else if(nextLine[153].equals("Waste Heat")){
          line[24]="refuse";
        }else if(nextLine[153].equals("Solar")){
          line[24]="solar";
        }else if(nextLine[153].equals("Hydro")){
          line[24]="hydro";
        }else if(nextLine[153].equals("Wood or Wood Waste")){
          line[24]="biomass";
        }else if(nextLine[153].equals("Distillate Fuel Oil (Diesel,FO1,FO2,FO4)")||nextLine[153].equals("Jet Fuel")||nextLine[153].equals("Residual Fuel Oil (FO5, FO6)")){
          line[24]="oil";
        }else{
          line[24]="unknown";
        }
      }else{
        if(nextLine[17].equals("ANT")||nextLine[17].equals("BIT")||nextLine[17].equals("LIG")||nextLine[17].equals("RC")||nextLine[17].equals("SUB")||nextLine[17].equals("WC")){
          line[24]="coal";
        }else if(nextLine[17].equals("NG")||nextLine[17].equals("PG")||nextLine[17].equals("SG")||nextLine[17].equals("SGC")){
          line[24]="ng";
        }else if(nextLine[17].equals("DFO")||nextLine[17].equals("RFO")||nextLine[17].equals("JF")||nextLine[17].equals("KER")||nextLine[17].equals("WO")||nextLine[17].equals("PC")){
          line[24]="oil";
        }else if(nextLine[17].equals("NUC")){
          line[24]="nuclear";
        }else if(nextLine[17].equals("WND")){
          line[24]="wind";
        }else if(nextLine[17].equals("SUN")){
          line[24]="solar";
        }else if(nextLine[17].equals("GEO")){
          line[24]="geothermal";
        }else if(nextLine[17].equals("MWH")){
          line[24]="storage";
        }else if(nextLine[17].equals("MSW")||nextLine[17].equals("SLW")||nextLine[17].equals("SG")||nextLine[17].equals("BFG")||nextLine[17].equals("OG")||nextLine[17].equals("TDF")||nextLine[17].equals("OTH")||nextLine[17].equals("WH")){
          line[24]="refuse";
        }else if(nextLine[17].equals("CUR")||nextLine[17].equals("WAT")||nextLine[17].equals("TID")||nextLine[17].equals("WV")){
          line[24]="hydro";
        }else if(nextLine[17].equals("AB")||nextLine[17].equals("OBS")||nextLine[17].equals("LFG")||nextLine[17].equals("OBG")||nextLine[17].equals("BLQ")||nextLine[17].equals("OBL")||nextLine[17].equals("WDL")||nextLine[17].equals("WDS")){
          line[24]="biomass";
        }else if(nextLine[17].equals("NA")||nextLine[17].equals("PUR")){
          line[24]="unknown";
        }else{
          line[24]="nongen";
        }      
      }
      
      //columns with all 0s or 2s
      line[25]="2";
      line[26]="0";
      line[27]="0";
      line[28]="2";
      
      //heat rate we use
      if(nextLine[483].equals("10.4")&&!nextLine[148].equals("")){
        nextLine[160]=nextLine[148];
      }
      else{
        nextLine[160]=nextLine[483];
      }
      line[38]=nextLine[160];
      
      //ENTER GAS PRICE per MMBTU (and average EV fuel price for that fuel type)
      Double gasprice = 5.5;
      
      //ENTER COAL PRICE per MMBTU (and average EV fuel price for that fuel type) 
      Double coalprice = 0.0;
      
      //ENTER OIL PRICE per MMBTU (and average EV fuel price for that fuel type)
      Double oilprice = 0.0;
      
      //VOM we use
      if(nextLine[151].equals("")){
        if(line[24].equals("ng")){
          nextLine[161]=avgOMgas+"";
        }
        else if(line[24].equals("coal")){
          nextLine[161]=avgOMcoal+"";
        }
        else if(line[24].equals("oil")){
          nextLine[161]=avgOMoil+"";
        }
        else {
          nextLine[161]="0.0";
        }
      }
      else{
        nextLine[161]=nextLine[152];
      }
      line[39]=nextLine[161];
      
      //Update column with exogenous fuel cost (only for ng)
      nextLine[158]="0";
      if(line[24].equals("ng")){
        //nextLine[158]=gasprice+"";
        if(nextLine[151].equals("")){
          nextLine[158]=avgEVgas+"";
        }else{
          nextLine[158]=nextLine[151];
        }
      }
      if(line[24].equals("coal")){
        //update to below if using exogenous coal price
        //nextLine[158]=coalprice+"";
        if(nextLine[151].equals("")){
          nextLine[158]=avgEVcoal+"";
        }else{
          nextLine[158]=nextLine[151];
        }
      }
      if(line[24].equals("oil")){
        //update to below if using exogenous oil price
        //nextLine[158]=oilprice+"";
        if(nextLine[151].equals("")){
          nextLine[158]=avgEVoil+"";
        }else{
          nextLine[158]=nextLine[151];
        }
      }
      if(!line[24].equals("ng")&&!line[24].equals("ng")&&!line[24].equals("ng")&&!nextLine[151].equals("")){
        nextLine[158]=nextLine[151];
      }
      line[37]=nextLine[158];
      
      //Update column with BaseCost sans gas coal and oil cost
      nextLine[159]=nextLine[161];
      
      if(!nextLine[148].equals("")){
        if(nextLine[483].equals("10.4")&&!nextLine[148].equals("")){
          if(line[24].equals("ng")){
            nextLine[159]=(Double.parseDouble(nextLine[152])+(Double.parseDouble(nextLine[151])-avgEVgas)*Double.parseDouble(nextLine[148]))+"";
          }
          if(line[24].equals("coal")){
            nextLine[159]=(Double.parseDouble(nextLine[152])+(Double.parseDouble(nextLine[151])-avgEVcoal)*Double.parseDouble(nextLine[148]))+"";
          }
          if(line[24].equals("oil")){
            nextLine[159]=(Double.parseDouble(nextLine[152])+(Double.parseDouble(nextLine[151])-avgEVoil)*Double.parseDouble(nextLine[148]))+"";
          }
        }else{
          if(line[24].equals("ng")){
            nextLine[159]=(Double.parseDouble(nextLine[152])+(Double.parseDouble(nextLine[151])-avgEVgas)*Double.parseDouble(nextLine[483]))+"";
          }
          if(line[24].equals("coal")){
            nextLine[159]=(Double.parseDouble(nextLine[152])+(Double.parseDouble(nextLine[151])-avgEVcoal)*Double.parseDouble(nextLine[483]))+"";
          }
          if(line[24].equals("oil")){
            nextLine[159]=(Double.parseDouble(nextLine[152])+(Double.parseDouble(nextLine[151])-avgEVoil)*Double.parseDouble(nextLine[483]))+"";
          }
        }
      }
      line[40]=nextLine[159];
      
      // calculate base cost
      line[29]="";
      if(line[24].equals("nuclear")||line[24].equals("wind")||line[24].equals("solar")||line[24].equals("geothermal")||line[24].equals("storage")||line[24].equals("refuse")||line[24].equals("hydro")||line[24].equals("biomass")||line[24].equals("unknown")){
        // base cost for biomass and unknown units
        if(line[24].equals("biomass")||line[24].equals("unknown")){
          if(nextLine[152].equals("")){
            line[29]="2";
          }else{
            line[29]=Double.parseDouble(nextLine[152])+Double.parseDouble(nextLine[151])*Double.parseDouble(nextLine[160])+"";
          }
        }
        // base cost for nuclear, wind, solar, geothermal, storage, refuse and hyro units
        else{
          line[29]="2";
        }
      }else{
        if(nextLine[151].equals("")){
          //base cost for natural gas units (if no EV Data)
          if(line[24].equals("ng")){
            line[29]=avgOMgas+Double.parseDouble(nextLine[158])*Double.parseDouble(nextLine[160])+"";
          }
          // base cost for coal units (if no EV Data)
          if(line[24].equals("coal")){
            line[29]=avgOMcoal+Double.parseDouble(nextLine[158])*Double.parseDouble(nextLine[160])+"";
          }
          // base cost for oil units (if no EV Data)
          if(line[24].equals("oil")){
            line[29]=avgOMoil+Double.parseDouble(nextLine[158])*Double.parseDouble(nextLine[160])+"";
          }
        }else{
          //base cost for natural gas units (if EV Data)
          if(line[24].equals("ng")){
            line[29]=Double.parseDouble(nextLine[152])+Double.parseDouble(nextLine[158])*Double.parseDouble(nextLine[160])+"";
          }else{
            // base cost for coal and oil units (if EV Data)
            line[29]=Double.parseDouble(nextLine[152])+Double.parseDouble(nextLine[151])*Double.parseDouble(nextLine[160])+"";
          }
        }
      }
      //if base cost is negative, replace by average of the fuel type
      if(Double.parseDouble(line[29])<0){
        if(line[24].equals("ng")){
          line[29]=avgEVgas+"";
        }
        if(line[24].equals("coal")){
          line[29]=avgEVcoal+"";
        }
        if(line[24].equals("oil")){
          line[29]=avgEVoil+"";
        }
      }
      
      line[30]="0";
      
      //co2
      line[31]="";
      Double mwh =0.0;
      if(!nextLine[100].equals("")&&!nextLine[105].equals("")&&!nextLine[105].equals("0")){
        mwh =Double.parseDouble(nextLine[100])/1.05;
        line[31]=(Double.parseDouble(nextLine[105])/mwh)+"";
      }else{
        // EIA co2 rate * heat rate
        if(line[24].equals("coal")){
          line[31]=Double.parseDouble(nextLine[483])*0.106351091+"";//coal
        }else if (line[24].equals("ng")){
          line[31]=Double.parseDouble(nextLine[483])*0.058540601+"";//gas
        }else if(line[24].equals("oil")){
          line[31]=Double.parseDouble(nextLine[483])*0.08382386+"";//oil
        }else{
          line[31]="0";
        }
      }
      //if unlikely large value replace by average
      if(Double.parseDouble(line[31])>3){
        line[31]=".44";
      }
      
      //nox
      line[32]=Double.parseDouble(nextLine[479])*Double.parseDouble(nextLine[483])+"";
      
      //so2
      line[33]=Double.parseDouble(nextLine[480])*Double.parseDouble(nextLine[483])+"";
      
      //env damages/mwh
      line[34]=nextLine[486];
      line[35]=nextLine[488];
      line[36]=nextLine[489];
      
      //name
      line[41]=nextLine[141]; 
      if(line[41].equals("")){
        line[41]=nextLine[5]; 
      }
      
      //nation
      if(!nextLine[3].equals("")){
        line[42]="US"; 
      }else if(nextLine[144].equals("BJ")){
        line[42]="Mexico"; 
      }else{
        line[42]="Canada"; 
      }
      
      //state or province
      if(!nextLine[6].equals("")){
        line[43]=nextLine[6]; 
      }else{
        line[43]=nextLine[144]; 
      }
      
      // Compute Interconnection
      reader2 = new CsvReader(new FileReader(ADDRESS_FILE2));
      Boolean found_interconnection = false;
      String [] interconnection = {""};
      while (reader2.readRecord()&&!found_interconnection){
        nextLine_interconnection_file = reader2.getValues(); 
        if(nextLine_interconnection_file[0].equals(line[1])){
          interconnection[0] = nextLine_interconnection_file[1];
          found_interconnection = true;
        }
      }
      
      // find closest US unit with emissions data if unit is in Canada
      reader_canada_search = new CsvReader(new FileReader(ADDRESS_FILE));
      reader_canada_search.readRecord();
      Double min_dist = 999999999.9; String [] closest_US_unit = {"",""}; 
      String [] nextLine_canada_search = {};
      if(nextLine[6].equals("")){
        while (reader_canada_search.readRecord()){
          nextLine_canada_search = reader_canada_search.getValues(); 
          if(!nextLine_canada_search[6].equals("")&&!nextLine_canada_search[157].equals("")){
            if(Double.parseDouble(nextLine_canada_search[489])>0){
              if(Double.parseDouble(nextLine_canada_search[36])>250.0&&Double.parseDouble(nextLine_canada_search[36])<500.0){
                if(dist(Double.parseDouble(nextLine_canada_search[157]),Double.parseDouble(nextLine[157]),
                        Double.parseDouble(nextLine_canada_search[156]),Double.parseDouble(nextLine[156]))<min_dist){
                  min_dist=dist(Double.parseDouble(nextLine_canada_search[89]),Double.parseDouble(nextLine[157]),
                        Double.parseDouble(nextLine_canada_search[90]),Double.parseDouble(nextLine[156]));
                  closest_US_unit[0]=nextLine_canada_search[4];
                  closest_US_unit[1]=nextLine_canada_search[8];
                }
              }
            }
          }
        }
        nextLine[4] = closest_US_unit[0];
        nextLine[8] = closest_US_unit[1];
      }
      
      nextLine[0]="";
      writer.writeRecord(concat(concat(interconnection,line),nextLine));
    }
    
    writer.flush();
    writer.close();
    
    //print results of match
    System.out.println("Done" );
    
    //switch so2 and nox columns
    reader = new CsvReader(new FileReader("EIA-EV_Gens_Ray.csv"));
    while (reader.readRecord()){
      nextLine = reader.getValues(); 
      String sw1 = nextLine[28];
      nextLine[28]=nextLine[27];
      nextLine[27]=sw1;
    }
  }
  //function to concatenate string arrays
  public static final String[] concat(String[] s1, String[] s2) {
    String[] erg = new String[s1.length + s2.length];
    System.arraycopy(s1, 0, erg, 0, s1.length);
    System.arraycopy(s2, 0, erg, s1.length, s2.length);
    return erg;
  }
  
  //function to calculate distance in KM between 2 coordinates
  public static final double dist(double lat1, double lat2, double lon1, double lon2) {
    int r = 6371; // km
    double rad = 0.0174532925;
    double dis = Math.acos(Math.sin(lat1*rad)*Math.sin(lat2*rad) + 
                           Math.cos(lat1*rad)*Math.cos(lat2*rad) *
                           Math.cos(lon2*rad-lon1*rad)) * r;
    
    return dis;
  }
}

