//**********************************************************
// Program: regression_finnish.java
//
// Author: Charles Marquet
//

// Description: This program prepares uses regression software output to add estimated emission and heat rates to the dataset
//              This program also checks the emission rates are reasonable
//
// Note: we hard-coded the emission rates for IC units             
//
// Input: The result_full.csv file, from the stata regression analysis
//        The EIA-EV_Gens_1.csv file
//
// Output: The EIA-EV-emissions_Gens.csv file, containing EIA-EPA-EV units with the estimated emission rates
//
// Need the free csv reader/writer from: http://sourceforge.net/projects/javacsv/
//*********************************************************

import java.io.*;
import java.lang.String;
import java.lang.Integer;
import com.csvreader.CsvReader;
import com.csvreader.CsvWriter;
import java.util.*;


public class regression_finnish_full {
  //these are the input file names
  //Input: EIA-EPA-stacks matched file
  private static final String ADDRESS_FILE="result_full.csv";
  private static final String ADDRESS_FILE1="EIA-EV_Gens_1.csv";
  
  public static void main(String[] args) throws IOException {
    
    //these read the input files
    CsvReader reader = new CsvReader(new FileReader(ADDRESS_FILE));
    CsvReader reader1 = new CsvReader(new FileReader(ADDRESS_FILE1));
    
    //initiate variables
    String [] nextLine;
    String [] nextLine1;
    
    //output file name
    CsvWriter writer = new CsvWriter("EIA-EV-emissions_Gens.csv");
    writer.setDelimiter(',');
    
    //write title
    reader1.readRecord();
    nextLine1 = reader1.getValues(); 
    String [] title = nextLine1;
    writer.writeRecord(title); 
    writer.flush();
    
    //create arrays to store maxes and mins of emission rates.
    Double [] maxes_so2 = new Double[70];
    Double [] maxes_nox = new Double[70];
    Double [] maxes_hr = new Double[70];
    Double [] mins_so2 = new Double[70];
    Double [] mins_nox = new Double[70];
    Double [] mins_hr = new Double[70];
    Arrays.fill(maxes_so2, 0.0);
    Arrays.fill(maxes_nox, 0.0);
    Arrays.fill(maxes_hr, 0.0);
    Arrays.fill(mins_so2, 10.0);
    Arrays.fill(mins_nox, 10.0);
    Arrays.fill(mins_hr, 100.0);
    
    //this loops through the regression data
    reader.readRecord();
    while (reader.readRecord()){
      //values from current line
      nextLine = reader.getValues(); 
      
      //units are grouped by fuel type and prime mover
      int group = Integer.parseInt(nextLine[117+9])-1;
      
      //for each group, find max and min emisson/heat rates
      try{if(maxes_so2[group]<Double.parseDouble(nextLine[9])){
        maxes_so2[group]=Double.parseDouble(nextLine[9]);
      };}catch(NumberFormatException e){};
      try{if(maxes_nox[group]<Double.parseDouble(nextLine[8])){
        maxes_nox[group]=Double.parseDouble(nextLine[8]);
      };}catch(NumberFormatException e){};
      try{if(maxes_hr[group]<Double.parseDouble(nextLine[10])){
        maxes_hr[group]=Double.parseDouble(nextLine[10]);
      };}catch(NumberFormatException e){};
      try{if(mins_so2[group]>Double.parseDouble(nextLine[9])){
        mins_so2[group]=Double.parseDouble(nextLine[9]);
      };}catch(NumberFormatException e){};
      try{if(mins_nox[group]>Double.parseDouble(nextLine[8])){
        mins_nox[group]=Double.parseDouble(nextLine[8]);
      };}catch(NumberFormatException e){};
      try{if(mins_hr[group]>Double.parseDouble(nextLine[10])){
        mins_hr[group]=Double.parseDouble(nextLine[10]);
      };}catch(NumberFormatException e){};
    }
    
    //check for errors in max/min arrays (if default values were not replaced)
    for(int j=0;j<70;j++){
      if(mins_so2[j]==10.0){
        mins_so2[j]=0.0;
        maxes_so2[j]=0.0;
      }
      if(mins_nox[j]==10.0){
        mins_nox[j]=0.0;
        maxes_nox[j]=0.0;
      }
      if(mins_hr[j]==100.0){
        mins_hr[j]=10.4;
        maxes_hr[j]=10.4;
      }
    }
    
    //edit emissions data
    reader = new CsvReader(new FileReader(ADDRESS_FILE));
    reader.readRecord();
    
    int i =0;
    //loop through dataset one generator at a time
    while (reader1.readRecord()){
      
      int l = reader1.getValues().length;
      String [] blank = new String[488-l];
      blank[487-l]="0";
      nextLine1 = concat(reader1.getValues(),blank);
      
      /*
      //EV unit with no EIA info
      Double blankeia = -1000.0;
      try{ blankeia = Double.parseDouble(nextLine1[11]);
      }
      catch(NumberFormatException z){
        nextLine1[480]="0.0";
        nextLine1[479]="0.0";
        nextLine1[483]="10.4";
      }
      if(blankeia!=-1000.0){ //unit with EIA info
      */
        reader.readRecord();
        nextLine = reader.getValues();
        int group = Integer.parseInt(nextLine[117+9])-1;
        i++;
   
        //calculate SO2 emission rate (try actual data, then estimated data, then default value)
        try{nextLine1[480]=Double.parseDouble(nextLine1[102])/Double.parseDouble(nextLine1[106])*2000+"";
        }catch(NumberFormatException e){
          try{nextLine1[480]=nextLine[129+9];
            if(Double.parseDouble(nextLine[129+9])>maxes_so2[group]){
              nextLine1[480]=maxes_so2[group]+"";
            }
            if(Double.parseDouble(nextLine[129+9])<mins_so2[group]){
              nextLine1[480]=mins_so2[group]+"";
            };}catch(NumberFormatException f){
              nextLine1[480]="0.0";
            }
        };
        //fix IC emission rates
        if(nextLine1[9].equals("IC")){
          if(nextLine1[88].equals("LIQUID")){
            nextLine1[480]="0.2";
          }
          if(nextLine1[88].equals("GAS")){
            nextLine1[480]="0.0";
          }
        }
        
        //calculate nox emission rate (try actual data, then estimated data, then default value)
        try{ nextLine1[479]=nextLine1[103];
        }catch(NumberFormatException e){
          try{nextLine1[479]=nextLine[127+9];
            if(Double.parseDouble(nextLine[127+9])>maxes_nox[group]){
              nextLine1[479]=maxes_nox[group]+"";
            }
            if(Double.parseDouble(nextLine[127+9])<mins_nox[group]){
              nextLine1[479]=mins_nox[group]+"";
            };}catch(NumberFormatException f){
              nextLine1[479]="0.0";
            }
        };
        //fix IC emission rates
        if(nextLine1[9].equals("IC")){
          if(nextLine1[88].equals("LIQUID")){
            nextLine1[479]="1.0";
          }
          if(nextLine1[88].equals("GAS")){
            nextLine1[479]="1.0";
          }
        }
        
        //calculate heat rate, "hr" (try actual data, then estimated data, then default value)
        try{nextLine1[483]=nextLine1[127];
        }catch(NumberFormatException e){
          try{nextLine1[483]=nextLine[131+9];
            if(Double.parseDouble(nextLine[131+9])>maxes_hr[group]){
              nextLine1[483]=maxes_hr[group]+"";
            }
            if(Double.parseDouble(nextLine[131+9])<mins_hr[group]){
              nextLine1[483]=mins_hr[group]+"";
            };}catch(NumberFormatException f){
              nextLine1[483]="10.4";
            }
        };
        
        //if heat rate was a default value, try getting actual or estimated value
        if(nextLine1[483].equals("10.4")||nextLine1[483].equals("7")){
          try{nextLine1[483]=nextLine[131+9];
            if(Double.parseDouble(nextLine[131+9])>maxes_hr[group]){
              nextLine1[483]=maxes_hr[group]+"";
            }
            if(Double.parseDouble(nextLine[131+9])<mins_hr[group]){
              nextLine1[483]=mins_hr[group]+"";
            };}catch(NumberFormatException f){
              nextLine1[483]="10.4";
            }
        }
        
        //deal with canadian units
        if(nextLine1[11].equals("")){
          //default
          nextLine1[480]="0.0";
          nextLine1[479]="0.0";
          nextLine1[483]="10.4";
          
          //coal
          if(nextLine1[153].equals("Coal")){
          nextLine1[480]="0.816";
          nextLine1[479]="0.262";
          nextLine1[483]="10.12";
          }
          
          //gas
          if(nextLine1[153].equals("Natural Gas")){
          nextLine1[480]="0.0104";
          nextLine1[479]="0.112";
          nextLine1[483]="10.84";
          }
          
          //biomass
          if(nextLine1[153].equals("Wood or Wood Waste")){
          nextLine1[480]="0.064";
          nextLine1[479]="0.152";
          nextLine1[483]="13.01";
          }
        }
      
      
      //write result to file
      writer.writeRecord(nextLine1);
    }
    
    writer.flush();
    writer.close();
    System.out.println("Done" );
  }
  //function to concatenate string arrays
  public static final String[] concat(String[] s1, String[] s2) {
    String[] erg = new String[s1.length + s2.length];
    System.arraycopy(s1, 0, erg, 0, s1.length);
    System.arraycopy(s2, 0, erg, s1.length, s2.length);
    return erg;
  }
}

